// const { rules: coreStylistic } = require('./core-stylistic-issues');
// const { rules: coreBestPractices } = require('./core-best-practices');

// ----------------------------------------------------------------------------
// Babel
// https://github.com/babel/eslint-plugin-babel
// ----------------------------------------------------------------------------

const rules = {
    // Ignores capitalized decorators (@Decorator)
    'new-cap': 'off',
    // 'babel/new-cap': coreStylistic['new-cap'],
    'babel/new-cap': ['error', { newIsCap: true, capIsNew: false }],

    // doesn't complain about optional chaining (var foo = bar?.a_b;`)
    camelcase: 'off',
    // 'babel/camelcase': coreStylistic['camelcase'],
    'babel/camelcase': [
        'error',
        {
            properties: 'always',
            ignoreDestructuring: false,
            // TODO enable:
            // ignoreImports: false,
            // allow: ['^UNSAFE_'],
        },
    ],

    // doesn't fail when inside class properties (`class A { a = this.b; }`)
    'no-invalid-this': 'off',
    'babel/no-invalid-this': 'error',

    // doesn't complain about export x from "mod"; or export * as x from "mod";
    'object-curly-spacing': 'off',
    // 'babel/object-curly-spacing': coreStylistic['object-curly-spacing'],
    'babel/object-curly-spacing': ['error', 'always'],

    // doesn't complain about JSX fragment shorthand syntax
    quotes: 'off',
    // 'babel/quotes': coreStylistic['quotes'],
    'babel/quotes': [
        // TODO
        'error',
        'single',
        {
            avoidEscape: true,
            allowTemplateLiterals: true,
        },
    ],

    // doesn't fail when using for await (let something of {}). Includes class properties
    semi: 'off',
    // 'babel/semi': coreStylistic['semi'],
    'babel/semi': ['error', 'always'],

    // doesn't fail when using do expressions
    'no-unused-expressions': 'off',
    // 'babel/no-unused-expressions': coreBestPractices['no-unused-expressions'],
    'babel/no-unused-expressions': [
        // TODO
        'error',
        {
            allowShortCircuit: false,
            allowTernary: false,
            allowTaggedTemplates: false,
        },
    ],

    // doesn't complain when used with BigInt (typeof BigInt(9007199254740991) === 'bigint')
    // 'valid-typeof': 'off',
    // 'babel/valid-typeof': 'off',
};

module.exports = {
    plugins: ['babel'],

    rules,
};
