// ----------------------------------------------------------------------------
// Best Practices
// ----------------------------------------------------------------------------

const rules = {
    // Enforces getter/setter pairs in objects
    'accessor-pairs': 'off', // TODO

    // Enforces return statements in callbacks of array's methods
    'array-callback-return': ['error', { allowImplicit: true }],

    // treat `var` statements as if they were block scoped
    'block-scoped-var': 'error',

    // enforce that class methods utilize this
    'class-methods-use-this': 'off', // TODO

    // specify the maximum cyclomatic complexity allowed in a program
    complexity: 'off', // TODO

    // require `return` statements to either always or never specify values
    'consistent-return': 'warn',

    // specify curly brace conventions for all control statements
    curly: ['error', 'all'], // TODO

    // require `default` case in `switch` statements
    'default-case': 'warn',

    // enforce default parameters to be last
    // REF typescript
    'default-param-last': 'off', // TODO

    // enforces consistent newlines before or after dots
    'dot-location': ['error', 'property'],

    // encourages use of dot notation whenever possible
    // REF typescript
    'dot-notation': ['error', { allowKeywords: true }], // TODO false

    // require the use of `===` and `!==`
    eqeqeq: ['error', 'always'],

    // require grouped accessor pairs in object literals and classes
    'grouped-accessor-pairs': ['error', 'getBeforeSet'],

    // make sure `for-in` loops have an `if` statement
    'guard-for-in': 'error',

    // enforce a maximum number of classes per file
    'max-classes-per-file': 'error',

    // disallow the use of `alert`, `confirm`, and `prompt`
    'no-alert': 'warn',

    // disallow use of `arguments.caller` or `arguments.callee`
    'no-caller': 'error',

    // disallow lexical declarations in case clauses
    'no-case-declarations': 'error', // TODO

    // disallow returning value from constructor
    'no-constructor-return': 'error',

    // disallow division operators explicitly at beginning of regular expression
    'no-div-regex': 'error', // TODO

    // disallow `else` after a `return` in an `if`
    'no-else-return': 'off', // TODO

    // disallow use of empty functions
    // REF typescript
    'no-empty-function': 'error',

    // disallow use of empty destructuring patterns
    'no-empty-pattern': 'error',

    // disallow comparisons to null without a type-checking operator
    'no-eq-null': 'error',

    // disallow use of `eval()`
    'no-eval': 'error',

    // disallow adding to native types
    'no-extend-native': 'error',

    // disallow unnecessary function binding
    'no-extra-bind': 'error',

    // disallow unnecessary labels
    'no-extra-label': 'error',

    // disallow fallthrough of `case` statements
    'no-fallthrough': 'error',

    // disallow the use of leading or trailing decimal points in numeric literals
    'no-floating-decimal': 'error',

    // disallow assignments to native objects or read-only global variables
    'no-global-assign': 'error',

    // disallow the type conversions with shorter notations
    'no-implicit-coercion': [
        // TODO off?
        'error',
        {
            boolean: true,
            number: true,
            string: true,
        },
    ],

    // disallow var and named functions in global scope
    'no-implicit-globals': 'error', // TODO

    // disallow use of `eval()`-like methods
    'no-implied-eval': 'error',

    // disallow `this` keywords outside of classes or class-like objects
    // REF babel
    // REF typescript
    'no-invalid-this': 'off', // TODO

    // disallow usage of `__iterator__` property
    'no-iterator': 'error',

    // disallow use of labeled statements
    'no-labels': [
        'error',
        {
            allowLoop: false,
            allowSwitch: false,
        },
    ],

    // disallow unnecessary nested blocks
    'no-lone-blocks': 'error',

    // disallow creation of functions within loops
    'no-loop-func': 'error',

    // disallow the use of magic numbers
    // REF typescript
    'no-magic-numbers': [
        // TODO
        'off',
        {
            ignoreArrayIndexes: true,
            enforceConst: true,
        },
    ],

    // disallow use of multiple spaces
    'no-multi-spaces': [
        // TODO
        'error',
        {
            exceptions: {
                Property: true,
                VariableDeclarator: true,
                ImportDeclaration: true,
            },
        },
    ],

    // disallow use of multiline strings
    'no-multi-str': 'error',

    // disallow use of new operator for `Function` object
    'no-new-func': 'error',

    // disallows creating new instances of `String`,`Number`, and `Boolean`
    'no-new-wrappers': 'error',

    // disallow use of the `new` operator when not part of an assignment or comparison
    'no-new': 'error',

    // disallow use of octal escape sequences in string literals, such as `var foo = "Copyright \251";`
    'no-octal-escape': 'error',

    // disallow use of octal literals
    'no-octal': 'error',

    // disallow reassignment of function parameters
    'no-param-reassign': ['warn', { props: false }], // props: true causes issues with immer (https://github.com/immerjs/immer/issues/189)

    // disallow usage of `__proto__` property
    'no-proto': 'error',

    // disallow declaring the same variable more than once
    'no-redeclare': 'error',

    // disallow certain properties on certain objects
    'no-restricted-properties': 'off', // TODO

    // disallow use of assignment in `return` statement
    'no-return-assign': ['error', 'except-parens'],

    // disallow unnecessary `return await`
    // REF typescript
    'no-return-await': 'error',

    // disallow use of `javascript:` urls.
    'no-script-url': 'error',

    // disallow assignments where both sides are exactly the same
    'no-self-assign': ['error', { props: true }],

    // disallow comparisons where both sides are exactly the same
    'no-self-compare': 'error',

    // disallow use of the comma operator
    'no-sequences': 'error',

    // restrict what can be thrown as an exception
    'no-throw-literal': 'error',

    // disallow unmodified conditions of loops
    'no-unmodified-loop-condition': 'off', // TODO

    // disallow usage of expressions in statement position
    // REF babel
    // REF typescript
    // REF chai-friendly
    'no-unused-expressions': [
        // TODO
        'error',
        {
            allowShortCircuit: false,
            allowTernary: false,
            allowTaggedTemplates: false,
        },
    ],

    // disallow unused labels
    'no-unused-labels': 'error',

    // disallow unnecessary `.call()` and `.apply()`
    'no-useless-call': 'error',

    // disallow unnecessary catch clauses
    'no-useless-catch': 'error',

    // disallow unnecessary concatenation of literals or template literals
    'no-useless-concat': 'error',

    // disallow unnecessary usage of escape character
    'no-useless-escape': 'error',

    // disallow redundant return statements
    'no-useless-return': 'error',

    // disallow use of the `void` operator
    'no-void': 'error',

    // disallow usage of configurable warning terms in comments
    'no-warning-comments': 'off', // TODO

    // disallow use of the `with` statement
    'no-with': 'error',

    // enforce using named capture group in regular expression
    'prefer-named-capture-group': 'off',

    // require using Error objects as Promise rejection reasons
    'prefer-promise-reject-errors': [
        // TODO
        'off',
        { allowEmptyReject: true },
    ],

    // disallow use of the RegExp constructor in favor of regular expression literals
    'prefer-regex-literals': 'off', // TODO

    // require use of the second argument for `parseInt()`
    radix: 'error',

    // disallow async functions which have no await expression
    // REF typescript
    'require-await': 'error',

    // enforce the use of u flag on RegExp
    'require-unicode-regexp': 'off',

    // require declaration of all vars at the top of their containing scope
    'vars-on-top': 'error', // TODO

    // require immediate function invocation to be wrapped in parentheses
    'wrap-iife': [
        'error',
        'any', // TODO
        { functionPrototypeMethods: false },
    ],

    // require or disallow Yoda conditions
    yoda: ['error', 'never'],
};

module.exports = {
    rules,
};
