// ----------------------------------------------------------------------------
// es6
// ----------------------------------------------------------------------------

const rules = {
    // require braces in arrow function body
    'arrow-body-style': ['warn', 'as-needed'], // TODO

    // require parens in arrow function arguments
    'arrow-parens': ['error', 'always'], // TODO

    // require space before/after arrow function's arrow
    'arrow-spacing': ['error', { before: true, after: true }],

    // verify calls of `super()` in constructors
    'constructor-super': 'error',

    // enforce spacing around the `*` in generator functions
    'generator-star-spacing': ['warn', { before: false, after: true }], // TODO

    // disallow modifying variables of class declarations
    'no-class-assign': 'error',

    // disallow arrow functions where they could be confused with comparisons
    'no-confusing-arrow': ['error', { allowParens: true }],

    // disallow modifying variables that are declared using `const`
    'no-const-assign': 'error',

    // disallow duplicate name in class members
    // REF typescript
    'no-dupe-class-members': 'error',

    // disallow duplicate module imports
    'no-duplicate-imports': ['error', { includeExports: true }],

    // disallow use of the `new` operator with the `Symbol` object
    'no-new-symbol': 'error',

    // restrict usage of specified modules when loaded by `import` declaration
    'no-restricted-imports': 'off',

    // disallow use of `this`/`super` before calling `super()` in constructors.
    'no-this-before-super': 'error',

    // disallow unnecessary computed property keys in object literals
    'no-useless-computed-key': 'error',

    // disallow unnecessary constructor
    // REF typescript
    'no-useless-constructor': 'error',

    // disallow renaming import, export, and destructured assignments to the same name
    'no-useless-rename': 'error',

    // require `let` or `const` instead of `var`
    'no-var': 'error',

    // require method and property shorthand syntax for object literals
    'object-shorthand': [
        // TODO
        'error',
        'always',
        {
            avoidQuotes: true,
            ignoreConstructors: false,
            avoidExplicitReturnArrows: true, // TODO
        },
    ],

    // suggest using arrow functions as callbacks
    // REF mocha
    'prefer-arrow-callback': 'error',

    // suggest using `const` declaration for variables that are never modified after declared
    'prefer-const': ['error', { destructuring: 'any', ignoreReadBeforeAssign: true }],

    // require destructuring from arrays and/or objects
    'prefer-destructuring': [
        'off', // TODO warn
        {
            VariableDeclarator: {
                array: false,
                object: true,
            },
            AssignmentExpression: {
                array: false,
                object: false,
            },
        },
        {
            enforceForRenamedProperties: true,
        },
    ],

    // disallow `parseInt()` in favor of binary, octal, and hexadecimal literals
    'prefer-numeric-literals': 'error',

    // suggest using the rest parameters instead of `arguments`
    'prefer-rest-params': 'error',

    // suggest using the spread operator instead of `.apply()`.
    'prefer-spread': 'error', // TODO

    // suggest using template literals instead of strings concatenation
    'prefer-template': 'error', // TODO

    // disallow generator functions that do not have `yield`
    'require-yield': 'off', // TODO

    // enforce spacing between rest and spread operators and their expressions
    'rest-spread-spacing': ['error', 'never'],

    // sort import declarations within module
    'sort-imports': 'off', // TODO

    // require symbol descriptions
    'symbol-description': 'warn',

    // enforce spacing around embedded expressions of template strings
    'template-curly-spacing': ['error', 'never'],

    // enforce spacing around the `*` in `yield*` expressions
    'yield-star-spacing': 'off', // TODO
};

module.exports = {
    rules,
};
