// plugins: ['import'],

// // settings:
// //   import/ignore:
// //     - 'node_modules'
// //     - '\\.(json|scss|css|jpg|png|gif|eot|otf|webp|svg|ttf|woff|woff2|mp4|webm)$'
// //   import/extensions:
// //     - '.js'
// //     - '.jsx'
// //   import/resolver:
// //     node:
// //       extensions:
// //         - '.js'
// //         - '.web.js'
// //         - '.android.js'
// //         - '.ios.js'
// //     webpack:
// //       config:
// //         resolve:
// //           extensions:
// //             - .js
// //             - .jsx

// ----------------------------------------------------------------------------
// Import
// https://github.com/benmosher/eslint-plugin-import
// ----------------------------------------------------------------------------

const rules = {
    'no-duplicate-imports': 'off',

    // ----------------------------------------------------------------------------
    // Static analysis
    // ----------------------------------------------------------------------------
    // Ensure imports point to a file/module that can be resolved.
    'import/no-unresolved': 'error', // TODO

    // Ensure named imports correspond to a named export in the remote file.
    'import/named': 'error',

    // Ensure a default export is present, given a default import.
    'import/default': 'off', // TODO

    // Ensure imported namespaces contain dereferenced properties as they are dereferenced.
    'import/namespace': ['error', { allowComputed: true }], // TODO

    // Restrict which files can be imported in a given folder
    'import/no-restricted-paths': 'off',

    // Forbid import of modules using absolute paths
    'import/no-absolute-path': 'error',

    // Forbid require() calls with expressions
    'import/no-dynamic-require': 'error',

    // Prevent importing the submodules of other modules
    'import/no-internal-modules': 'off',

    // Forbid Webpack loader syntax in imports
    'import/no-webpack-loader-syntax': 'error',

    // Forbid a module from importing itself
    'import/no-self-import': 'error',

    // Forbid a module from importing a module with a dependency path back to itself
    'import/no-cycle': 'off', // TODO warn?

    // Prevent unnecessary path segments in import and require statements
    'import/no-useless-path-segments': ['error', { commonjs: true }],

    // Forbid importing modules from parent directories
    'import/no-relative-parent-imports': 'off',

    // Ensures that modules contain exports and/or all modules are consumed within other modules
    'import/no-unused-modules': [
        // TODO
        'off',
        {
            missingExports: true,
            unusedExports: true,
        },
    ],

    // ----------------------------------------------------------------------------
    // Helpful warnings
    // ----------------------------------------------------------------------------
    // Report any invalid exports, i.e. re-export of the same name
    'import/export': 'error',

    // Report use of exported name as identifier of default export
    'import/no-named-as-default': 'error',

    // Report use of exported name as property of default export
    'import/no-named-as-default-member': 'error',

    // Report imported names marked with @deprecated documentation tag
    'import/no-deprecated': 'warn', // TODO

    // Forbid the use of extraneous packages
    'import/no-extraneous-dependencies': [
        // TODO
        'off',
        {
            devDependencies: true,
            optionalDependencies: true,
            peerDependencies: false,
        },
    ],

    // Forbid the use of mutable exports with var or let.
    'import/no-mutable-exports': 'error',

    // ----------------------------------------------------------------------------
    // Module systems
    // ----------------------------------------------------------------------------
    // Report potentially ambiguous parse goal (script vs. module)
    'import/unambiguous': 'off',

    // Report CommonJS require calls and module.exports or exports.*
    'import/no-commonjs': 'off',

    // Report AMD require and define calls.
    'import/no-amd': 'off',

    // No Node.js builtin modules.
    'import/no-nodejs-modules': 'off',

    // ----------------------------------------------------------------------------
    // Style Guide
    // ----------------------------------------------------------------------------
    // Ensure all imports appear before other statements
    'import/first': 'error',

    // Ensure all exports appear after other statements
    'import/exports-last': 'off', // TODO

    // Report repeated import of the same module in multiple places
    'import/no-duplicates': 'error',

    // Report namespace imports
    'import/no-namespace': 'off',

    // Ensure consistent use of file extension within the import path
    'import/extensions': [
        // TODO
        'error',
        'ignorePackages', // always except package imports
        {
            js: 'never',
            mjs: 'never',
            jsx: 'never',
            mjs: 'never',
            ts: 'never',
            tsx: 'never',
            // json: 'always',
            // svg: 'always',
            // png: 'always',
            // jpg: 'always',
            // ico: 'always',
            // graphql: 'always',
            // css: 'always',
            // sass: 'always',
            // scss: 'always',
        },
    ],

    // Enforce a convention in module import order
    'import/order': [
        // TODO
        'warn',
        {
            // groups: ['builtin', 'external', 'internal', 'parent', 'sibling'],
            groups: [
                ['builtin', 'external'],
                ['internal', 'parent', 'sibling', 'index'],
            ],
            'newlines-between': 'always-and-inside-groups', // TODO ignore, always, or always-and-inside-groups
            alphabetize: {
                order: 'ignore', // TODO asc?
                caseInsensitive: true,
            },
        },
    ],

    // Enforce a newline after import statements
    'import/newline-after-import': 'error',

    // Prefer a default export if module exports a single name
    'import/prefer-default-export': 'off', // TODO warn

    // Limit the maximum number of dependencies a module can have
    'import/max-dependencies': 'off',

    // Forbid unassigned imports
    'import/no-unassigned-import': 'off',

    // Forbid named default exports
    'import/no-named-default': 'error',

    // Forbid default exports
    'import/no-default-export': 'off',

    // Forbid anonymous values as default exports
    'import/no-anonymous-default-export': 'error',

    // Prefer named exports to be grouped together in a single export declaration
    'import/group-exports': 'off',

    // Enforce a leading comment with the webpackChunkName for dynamic imports
    'import/dynamic-import-chunkname': 'off', // TODO
};

module.exports = {
    plugins: ['import'],

    rules,
};
