// ----------------------------------------------------------------------------
// Mocha
// https://github.com/lo1tuma/eslint-plugin-mocha
// ----------------------------------------------------------------------------

const rules = {
    // enforces handling of callbacks for async tests
    'mocha/handle-done-callback': 'error',

    // limit the number of top-level suites in a single file
    'mocha/max-top-level-suites': ['error', { limit: 1 }],

    // disallow async functions passed to describe
    'mocha/no-async-describe': 'error',

    // disallow exclusive mocha tests
    'mocha/no-exclusive-tests': 'error',

    // disallow global tests
    'mocha/no-global-tests': 'error',

    // disallow hooks
    'mocha/no-hooks': 'off',

    // disallow hooks for a single test or test suite
    'mocha/no-hooks-for-single-case': 'off',

    // disallow identical titles
    'mocha/no-identical-title': 'error',

    // disallow arrow functions as arguments to mocha globals
    'mocha/no-mocha-arrows': 'off',

    // disallow tests to be nested within other tests
    'mocha/no-nested-tests': 'error',

    // disallow pending/unimplemented mocha tests
    'mocha/no-pending-tests': 'warn',

    // disallow returning in a test or hook function that uses a callback
    'mocha/no-return-and-callback': 'error',

    // disallow returning from an async test or hook
    'mocha/no-return-from-async': 'error',

    // disallow calling functions and dot operators directly in describe blocks
    'mocha/no-setup-in-describe': 'error',

    // disallow duplicate uses of a hook at the same level inside a describe
    'mocha/no-sibling-hooks': 'error',

    // disallow skipped mocha tests
    'mocha/no-skipped-tests': 'error',

    // disallow synchronous tests
    'mocha/no-synchronous-tests': ['error', { allowed: ['async', 'callback', 'promise'] }],

    // disallow top-level hooks
    'mocha/no-top-level-hooks': 'error',

    // prefer arrow function callbacks (mocha-aware)
    'prefer-arrow-callback': 'off',
    'mocha/prefer-arrow-callback': [
        'error',
        { allowNamedFunctions: false, allowUnboundThis: false },
    ],

    // match suite descriptions against a pre-configured regular expression
    'mocha/valid-suite-description': 'off',

    // match test descriptions against a pre-configured regular expression
    'mocha/valid-test-description': 'off',
};

module.exports = {
    plugins: ['mocha'],

    env: {
        mocha: true,
    },

    overrides: [
        {
            files: ['*.*{spec,test}.{js,ts,tsx}', '**/__{mocks,tests}__/**/*.{js,ts,tsx}'],
            rules,
        },
    ],
};
