// ----------------------------------------------------------------------------
// Node
// https://github.com/mysticatea/eslint-plugin-node
// ----------------------------------------------------------------------------

const rules = {
    // ensure Node.js-style error-first callback pattern is followed
    'node/no-callback-literal': 'error',

    // disallow the assignment to exports
    'node/no-exports-assign': 'error',

    // Disallow import declarations of extraneous packages
    'node/no-extraneous-import': 'off', // defer to import/no-extraneous-dependencies

    // Disallow require() expressions of extraneous packages
    'node/no-extraneous-require': 'off', // defer to import/no-extraneous-dependencies

    // Disallow import and export declarations for files that don't exist
    'node/no-missing-import': 'off',

    // Disallow require()s for files that don't exist.
    'node/no-missing-require': 'off',

    // Disallow bin files that npm ignores.
    'node/no-unpublished-bin': 'error',

    // Disallow import and export declarations for files that are not published.
    'node/no-unpublished-import': 'off',

    // Disallow require()s for files that are not published.
    'node/no-unpublished-require': 'off',

    // disallow unsupported ECMAScript built-ins on the specified version
    'node/no-unsupported-features/es-builtins': 'warn',

    // disallow unsupported ECMAScript syntax on the specified version
    'node/no-unsupported-features/es-syntax': 'warn',

    // disallow unsupported Node.js built-in APIs on the specified version
    'node/no-unsupported-features/node-builtins': 'error',

    // Make the same code path as throw at process.exit()
    'node/process-exit-as-throw': 'warn',

    // Suggest correct usage of shebang.
    'node/shebang': 'error',

    // Disallow deprecated APIs
    'node/no-deprecated-api': 'error',

    // Enforce either module.exports or exports.
    'node/exports-style': ['error', 'module.exports'],

    // enforce the style of file extensions in import declarations
    'node/file-extension-in-import': 'off', // todo

    // enforce either Buffer or require("buffer").Buffer
    'node/prefer-global/buffer': 'error',

    // enforce either console or require("console")
    'node/prefer-global/console': 'error',

    // enforce either process or require("process")
    'node/prefer-global/process': 'error',

    // enforce either TextDecoder or require("util").TextDecoder
    'node/prefer-global/text-decoder': 'error',

    // enforce either TextEncoder or require("util").TextEncoder
    'node/prefer-global/text-encoder': 'error',

    // enforce either URLSearchParams or require("url").URLSearchParams
    'node/prefer-global/url-search-params': 'error',

    // enforce either URL or require("url").URL
    'node/prefer-global/url': 'error',

    // enforce require("dns").promises
    'node/prefer-promises/dns': 'error', // TODO

    // enforce require("fs").promises
    'node/prefer-promises/fs': 'error', // TODO
};

module.exports = {
    plugins: ['node'],

    rules,
};
