// ----------------------------------------------------------------------------
// React Native a11y
// https://github.com/FormidableLabs/eslint-plugin-react-native-a11y
// ----------------------------------------------------------------------------

const rules = {
    // ----------------------------------------------------------------------------
    // Basic
    // ----------------------------------------------------------------------------

    // Enforce accessibilityHint is used in conjunction with accessibilityLabel
    'react-native-a11y/has-accessibility-hint': 'error',

    // Enforce that <Touchable\*> components only have either the accessibilityRole prop or both accessibilityTraits and accessibilityComponentType props set
    'react-native-a11y/has-accessibility-props': 'error',

    // Enforce both accessibilityActions and onAccessibilityAction props are valid
    'react-native-a11y/has-valid-accessibility-actions': 'error',

    // Enforce accessibilityRole property value is valid
    'react-native-a11y/has-valid-accessibility-role': 'error',

    // Enforce accessibilityState property value is valid
    'react-native-a11y/has-valid-accessibility-state': 'error',

    // Enforce accessibilityStates property value is valid
    'react-native-a11y/has-valid-accessibility-states': 'error',

    // Enforce accessibilityComponentType property value is valid
    'react-native-a11y/has-valid-accessibility-component-type': 'error', // TODO accessibilityComponentType will be deprecated

    // Enforce accessibilityTraits and accessibilityComponentType prop values must be valid
    'react-native-a11y/has-valid-accessibility-traits': 'error', // TODO accessibilityTraits will be deprecated

    // Enforce accessibilityValue property value is valid
    'react-native-a11y/has-valid-accessibility-value': 'error',

    // Enforce if a view has accessible={true}, that there are no touchable elements inside
    'react-native-a11y/no-nested-touchables': 'error',

    // ----------------------------------------------------------------------------
    // iOS
    // ----------------------------------------------------------------------------

    // Enforce that certain elements use accessibilityIgnoresInvertColors to avoid being inverted by device color settings
    'react-native-a11y/has-valid-accessibility-ignores-invert-colors': 'error',

    // ----------------------------------------------------------------------------
    // Android
    // ----------------------------------------------------------------------------

    // Enforce accessibilityLiveRegion prop values must be valid
    'react-native-a11y/has-valid-accessibility-live-region': 'error',

    // Enforce importantForAccessibility property value is valid
    'react-native-a11y/has-valid-important-for-accessibility': 'error',
};

module.exports = {
    plugins: ['react-native-a11y'],

    rules,
};
