// ----------------------------------------------------------------------------
// Testing Library
// https://github.com/testing-library/eslint-plugin-testing-library
// ----------------------------------------------------------------------------

const rules = {
    // Enforce async queries to have proper await
    'testing-library/await-async-query': 'error',

    // Enforce async utils to be awaited properly
    'testing-library/await-async-utils': 'error',

    // Enforce async fire event methods to be awaited
    'testing-library/await-fire-event': 'off',

    // Ensure data-testid values match a provided regex
    'testing-library/consistent-data-testid': 'off',

    // Disallow unnecessary await for sync queries
    'testing-library/no-await-sync-query': 'error',

    // Disallow the use of debug
    'testing-library/no-debug': 'error', // TODO {'renderFunctions':['renderWithRedux', 'renderWithRouter']}]

    // Disallow importing from DOM Testing Library
    'testing-library/no-dom-import': ['error', 'react'],

    // Disallow the use of cleanup
    'testing-library/no-manual-cleanup': 'error',

    // Disallow empty callbacks for waitFor and waitForElementToBeRemoved
    'testing-library/no-wait-for-empty-callback': 'error',

    // Suggest using explicit assertions rather than just getBy* queries
    'testing-library/prefer-explicit-assert': 'warn', // TODO

    // Enforce specific queries when checking element is present or not
    'testing-library/prefer-presence-queries': 'error',

    // Suggest using screen while using queries
    'testing-library/prefer-screen-queries': 'error',

    // Use waitFor instead of deprecated wait methods
    'testing-library/prefer-wait-for': 'error',
};

module.exports = {
    plugins: ['testing-library'],

    overrides: [
        {
            files: ['*.*{spec,test}.{js,ts,tsx}', '**/__{mocks,tests}__/**/*.{js,ts,tsx}'],
            rules,
        },
    ],
};
