const rules = {
    // ----------------------------------------------------------------------------
    // React
    // https://github.com/yannickcr/eslint-plugin-react
    // ----------------------------------------------------------------------------

    // Enforces consistent naming for boolean props
    'react/boolean-prop-naming': 'off',

    // Forbid "button" element without an explicit "type" attribute
    'react/button-has-type': 'error',

    // Prevent extraneous defaultProps on components
    'react/default-props-match-prop-types': 'error',

    // Rule enforces consistent usage of destructuring assignment in component
    'react/destructuring-assignment': 'off', // TODO

    // Prevent missing displayName in a React component definition
    'react/display-name': 'off',

    // Forbid certain props on Components
    'react/forbid-component-props': 'off',

    // Forbid certain props on DOM Nodes
    'react/forbid-dom-props': 'off',

    // Forbid certain elements
    'react/forbid-elements': 'off',

    // Forbid foreign propTypes
    'react/forbid-foreign-prop-types': 'error',

    // Forbid certain propTypes (any, array, object)
    'react/forbid-prop-types': 'warn',

    // Standardize the way function component get defined
    'react/function-component-definition': [
        'error',
        {
            namedComponents: 'arrow-function',
            unnamedComponents: 'function-expression',
        },
    ],

    // Prevent using this.state inside this.setState
    'react/no-access-state-in-setstate': 'warn', // TODO

    // Prevent adjacent inline elements not separated by whitespace
    'react/no-adjacent-inline-elements': 'warn',

    // Prevent using Array index in `key` props
    'react/no-array-index-key': 'error',

    // Prevent passing children as props
    'react/no-children-prop': 'error',

    // Prevent usage of dangerous JSX properties
    'react/no-danger': 'warn',

    // Prevent problem with children and props.dangerouslySetInnerHTML
    'react/no-danger-with-children': 'error',

    // Prevent usage of deprecated methods
    'react/no-deprecated': 'error',

    // Prevent usage of setState in componentDidMount
    'react/no-did-mount-set-state': [
        'error', // TODO
        // 'warn',
        // 'disallow-in-func',
    ],

    // Prevent usage of setState in componentDidUpdate
    'react/no-did-update-set-state': [
        'error', // TODO
        // 'warn',
        // 'disallow-in-func',
    ],

    // Prevent direct mutation of this.state
    'react/no-direct-mutation-state': 'error',

    // Prevent usage of findDOMNode
    'react/no-find-dom-node': 'error',

    // Prevent usage of isMounted
    'react/no-is-mounted': 'error',

    // Prevent multiple component definition per file
    'react/no-multi-comp': ['warn', { ignoreStateless: true }], // TODO off?

    // Prevent usage of shouldComponentUpdate when extending React.PureComponent
    'react/no-redundant-should-component-update': 'error',

    // Prevent usage of the return value of React.render
    'react/no-render-return-value': 'error',

    // Prevent usage of setState
    'react/no-set-state': 'off',

    // Prevent common casing typos
    'react/no-typos': 'error',

    // Prevent using string references
    'react/no-string-refs': ['error', { noTemplateLiterals: true }],

    // Prevent using this in stateless functional components
    'react/no-this-in-sfc': 'error',

    // Prevent invalid characters from appearing in markup
    'react/no-unescaped-entities': 'error',

    // Prevent usage of unknown DOM property
    'react/no-unknown-property': 'error',

    // Prevent usage of UNSAFE_ methods
    'react/no-unsafe': 'error',

    // Prevent definitions of unused prop types
    'react/no-unused-prop-types': ['error', { skipShapeProps: true }], // TODO

    // Prevent definitions of unused state properties
    'react/no-unused-state': 'error',

    // Prevent usage of setState in componentWillUpdate
    'react/no-will-update-set-state': 'error', // TODO

    // Require ES6 class declarations over React.createClass
    'react/prefer-es6-class': 'error',

    // Enforce that props are read-only
    'react/prefer-read-only-props': 'error',

    // Require stateless functions when not using lifecycle methods, setState or ref
    'react/prefer-stateless-function': ['warn', { ignorePureComponents: true }], // TODO

    // Prevent missing props validation in a React component definition
    'react/prop-types': 'warn',

    // Prevent missing React when using JSX
    'react/react-in-jsx-scope': 'error',

    // Enforce a defaultProps definition for every prop that is not a required prop
    'react/require-default-props': [
        'warn',
        { forbidDefaultForRequired: true, ignoreFunctionalComponents: false },
    ],

    // Enforce React components to have a shouldComponentUpdate method
    'react/require-optimization': 'off', // TODO

    // Enforce ES5 or ES6 class for returning value in render function
    'react/require-render-return': 'error',

    // Prevent extra closing tags for components without children
    'react/self-closing-comp': ['error', { component: true, html: true }],

    // Enforce component methods order
    'react/sort-comp': [
        // TODO
        'error',
        {
            order: [
                'type-annotations',
                'static-variables',
                'static-methods',
                'instance-variables',
                'lifecycle',
                // - '/^on.+$/'
                // - 'getters',
                // - 'setters',
                'instance-methods',
                'everything-else',
                '/^render.+$/',
                'render',
            ],
        },
    ],

    // Enforce propTypes declarations alphabetical sorting
    'react/sort-prop-types': [
        // TODO
        'warn',
        {
            callbacksLast: false,
            ignoreCase: true,
            requiredFirst: false,
            sortShapeProp: true,
        },
    ],

    // Enforce the state initialization style to be either in a constructor or with a class property
    'react/state-in-constructor': ['error', 'never'],

    // Enforces where React component static properties should be positioned.
    'react/static-property-placement': [
        'error',
        'property assignment',
        {
            childContextTypes: 'property assignment',
            contextTypes: 'property assignment',
            contextType: 'property assignment',
            defaultProps: 'static public field', // TODO
            displayName: 'property assignment',
            propTypes: 'property assignment',
        },
    ],

    // Enforce style prop value being an object
    'react/style-prop-object': 'error',

    // Prevent void DOM elements (e.g. <img />, <br />) from receiving children
    'react/void-dom-elements-no-children': 'error',

    // ----------------------------------------------------------------------------
    // JSX Specific
    // ----------------------------------------------------------------------------

    // Enforce boolean attributes notation in JSX
    'react/jsx-boolean-value': ['warn', 'never'], // TODO

    // Enforce or disallow spaces inside of curly braces in JSX attributes and expressions
    'react/jsx-child-element-spacing': 'off', // TODO

    // Validate closing bracket location in JSX
    'react/jsx-closing-bracket-location': ['error', 'line-aligned'], // TODO

    // Validate closing tag location in JSX
    'react/jsx-closing-tag-location': 'off', // TODO breaks indent

    // Enforce curly braces or disallow unnecessary curly braces in JSX
    'react/jsx-curly-brace-presence': ['error', 'never'], // TODO

    // Enforce or disallow newlines inside of curly braces in JSX attributes and expressions (fixable)
    'react/jsx-curly-newline': [
        'error',
        {
            multiline: 'consistent',
            singleline: 'consistent',
        },
    ],

    // Enforce or disallow spaces inside of curly braces in JSX attributes
    'react/jsx-curly-spacing': [
        'error',
        {
            when: 'never',
            children: true,
            allowMultiline: true,
        },
    ],

    // Enforce or disallow spaces around equal signs in JSX attributes
    'react/jsx-equals-spacing': ['error', 'never'],

    // Restrict file extensions that may contain JSX
    'react/jsx-filename-extension': [
        'error',
        {
            extensions: ['.js', '.jsx'],
        },
    ],

    // Enforce position of the first prop in JSX
    'react/jsx-first-prop-new-line': ['error', 'multiline-multiprop'], // TODO

    // Enforce shorthand or standard form for React fragments
    'react/jsx-fragments': ['error', 'syntax'], // TODO

    // Enforce or disallow spaces inside of curly braces in JSX attributes
    'react/jsx-handler-names': [
        'warn',
        {
            // TODO
            checkLocalVariables: false,
            eventHandlerPrefix: 'handle',
            eventHandlerPropPrefix: 'on',
        },
    ],

    // Validate JSX indentation
    'react/jsx-indent': ['error', 4],

    // Validate props indentation in JSX
    'react/jsx-indent-props': ['error', 4], // TODO first

    // Validate JSX has key prop when in array or iterator
    'react/jsx-key': 'error',

    // Validate JSX maximum depth
    'react/jsx-max-depth': 'off',

    // Limit maximum of props on a single line in JSX
    'react/jsx-max-props-per-line': ['error', { maximum: 1, when: 'always' }], // TODO

    // Prevent usage of .bind() and arrow functions in JSX props
    'react/jsx-no-bind': [
        'error',
        {
            ignoreRefs: true,
            allowArrowFunctions: true,
            allowFunctions: false,
            allowBind: false,
            ignoreDOMComponents: true,
        },
    ],

    // Prevent comments from being inserted as text nodes
    'react/jsx-no-comment-textnodes': 'error',

    // Prevent duplicate props in JSX
    'react/jsx-no-duplicate-props': ['error', { ignoreCase: true }],

    // Prevent usage of unwrapped JSX strings
    'react/jsx-no-literals': 'off',

    // Forbid javascript: URLs
    'react/jsx-no-script-url': [
        'error',
        [
            {
                name: 'Link',
                props: ['to'],
            },
        ],
    ],

    // Forbid target="_blank" attribute without rel="noreferrer"
    'react/jsx-no-target-blank': ['error', { allowReferrer: false, enforceDynamicLinks: 'always' }],

    // Disallow undeclared variables in JSX
    'react/jsx-no-undef': ['error', { allowGlobals: true }],

    // Disallow unnecessary fragments
    'react/jsx-no-useless-fragment': 'error',

    // Limit to one expression per line in JSX
    'react/jsx-one-expression-per-line': 'off', // TODO

    // Enforce PascalCase for user-defined JSX components
    'react/jsx-pascal-case': ['error', { allowAllCaps: true }],

    // Disallow multiple spaces between inline JSX props
    'react/jsx-props-no-multi-spaces': 'error',

    // Disallow JSX props spreading
    'react/jsx-props-no-spreading': [
        'warn',
        {
            html: 'enforce',
            custom: 'ignore', // TODO enforce?
            explicitSpread: 'ignore',
            exceptions: [],
        },
    ],

    // Enforce default props alphabetical sorting
    'react/jsx-sort-default-props': ['warn', { ignoreCase: true }],

    // Enforce props alphabetical sorting
    'react/jsx-sort-props': [
        // TODO
        'warn',
        {
            ignoreCase: true,
            callbacksLast: false,
            shorthandFirst: false,
            shorthandLast: true,
            noSortAlphabetically: true,
            reservedFirst: true,
        },
    ],

    // Validate whitespace in and around the JSX opening and closing brackets
    'react/jsx-tag-spacing': [
        'error',
        {
            closingSlash: 'never',
            beforeSelfClosing: 'always',
            afterOpening: 'never',
            beforeClosing: 'allow',
        },
    ],

    // Prevent React to be incorrectly marked as unused
    'react/jsx-uses-react': 'error',

    // Prevent variables used in JSX to be incorrectly marked as unused
    'react/jsx-uses-vars': 'error',

    // Prevent missing parentheses around multilines JSX
    'react/jsx-wrap-multilines': [
        // TODO
        'error',
        {
            declaration: 'parens-new-line',
            assignment: 'parens-new-line',
            return: 'parens-new-line',
            arrow: 'parens-new-line',
            condition: 'ignore',
            logical: 'ignore',
            prop: 'ignore',
        },
    ],
};

module.exports = {
    plugins: ['react'],
};
